package gate

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-device/errors"
	dbmodel "property-device/model"
	pb_v1 "property-device/pb/v1"
)

func checkGateDelParam(req *pb_v1.GateDelRequest) error {
	switch {
	case req.DeviceId == 0:
		return status.Error(10003, "设备id不能为空")
	}
	return nil
}

func GateDel(ctx context.Context, req *pb_v1.GateDelRequest) (reply *pb_v1.GateDelReply, err error) {

	reply = &pb_v1.GateDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = checkGateDelParam(req)
	if err != nil {
		return nil, err
	}
	p := &dbmodel.TGate{}
	where := [][2]interface{}{}
	where = dbmodel.WhereAdd(where, "id", req.DeviceId)
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if p.GardenId > 0 {
		return nil, status.Error(10003, "请先回收设备再删除")
	}

	err = p.Delete(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
