package gate

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-device/errors"
	dbmodel "property-device/model"
	pb_v1 "property-device/pb/v1"
	"time"
)

func checkGateEnableParam(req *pb_v1.GateEnableRequest) error {
	switch {
	case req.DeviceId == 0:
		return status.Error(10003, "设备id不能为空")
	case req.GardenId == 0:
		return status.Error(10003, "小区id不能为空")
	}
	return nil
}

func GateEnable(ctx context.Context, req *pb_v1.GateEnableRequest) (reply *pb_v1.GateEnableReply, err error) {

	reply = &pb_v1.GateEnableReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = checkGateEnableParam(req)
	if err != nil {
		return nil, err
	}
	p := &dbmodel.TGate{}
	where := [][2]interface{}{}
	where = dbmodel.WhereAdd(where, "id", req.DeviceId)
	where = dbmodel.WhereAdd(where, "garden_id", req.GardenId)
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	enable := int32(1)
	if !req.Enable {
		enable = 2
	}
	now := time.Now()
	values := map[string]interface{}{
		"enable":     enable,
		"updated_at": now,
	}

	err = p.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}
	delGateInfoFromRedis(req.DeviceId, "", 0)
	delGateInfoFromRedis(0, p.Sn, p.Protocol)

	return reply, nil
}
