package gate_record

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-device/errors"
	"property-device/model"
	pb_v1 "property-device/pb/v1"
)

func checkGateRecordListParam(req *pb_v1.GateRecordListRequest) error {
	switch {
	case req.DeviceId == "":
		return status.Error(10003, "设备id不能为空")
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	return nil
}

func getGateRecordSql(req *pb_v1.GateRecordListRequest) (string, string) {
	//layout := "2006-01-02 15:04:05"
	sql := fmt.Sprintf("select * from \"%s\" ", req.DeviceId)
	where := ""
	whereArray := []string{}
	if req.Start > 0 {
		whereArray = append(whereArray, fmt.Sprintf("open_time >= %d ", req.Start))
	}
	if req.End > 0 {
		whereArray = append(whereArray, fmt.Sprintf("open_time < %d ", req.End))
	}
	if req.IsVisitor > 0 {
		whereArray = append(whereArray, fmt.Sprintf("is_visitor = %d ", req.IsVisitor))
	}
	for _, v := range whereArray {
		if where == "" {
			where = fmt.Sprintf("where %s", v)
			continue
		}
		where = fmt.Sprintf("%s and %s", where, v)
	}

	offset := int64(req.Page-1) * int64(req.PageSize)
	countSql := fmt.Sprintf("select count(garden_id) as count from \"%s\" %s tz('Asia/Shanghai')", req.DeviceId, where)
	sql = fmt.Sprintf("%s %s order by time desc limit %d offset %d tz('Asia/Shanghai')", sql, where, req.PageSize, offset)
	fmt.Printf("sql:%s\n", sql)
	return sql, countSql
}

func getGateRecordCount(dbname string, sql string) (int64, error) {
	type Result struct {
		Count int64
	}
	array := []Result{}
	_, err := model.Query(sql, dbname, &array)
	if err != nil {
		return 0, errors.DataBaseError
	}
	//fmt.Printf("aaaaa:%s,%s,%v\n", sql, dbname, array)
	if len(array) == 0 {
		return 0, nil
	}
	return array[0].Count, nil
}

func GateRecordList(ctx context.Context, req *pb_v1.GateRecordListRequest) (reply *pb_v1.GateRecordListReply, err error) {

	reply = &pb_v1.GateRecordListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = checkGateRecordListParam(req)
	if err != nil {
		return nil, err
	}
	sql, countSql := getGateRecordSql(req)

	reply.Total, err = getGateRecordCount(model.GateDbname, countSql)
	if err != nil {
		return nil, err
	}
	reply.Page = req.Page
	if reply.Total == 0 {
		return reply, nil
	}

	list := []*pb_v1.GateRecordItem{}
	_, err = model.Query(sql, model.GateDbname, &list)
	if err != nil {
		return nil, err
	}
	reply.List = list

	return reply, nil
}
