package visitor

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-device/errors"
	dbmodel "property-device/model"
	pb_v1 "property-device/pb/v1"
)

func checkGateVisitorDelParam(req *pb_v1.GateVisitorDelRequest) error {
	switch {
	case req.Id == 0:
		return status.Error(10003, "id不能为空")
	case !req.Manager && req.Uid == 0:
		return status.Error(10003, "用户不能为空")

	}

	return nil
}

func GateVisitorDel(ctx context.Context, req *pb_v1.GateVisitorDelRequest) (reply *pb_v1.GateVisitorDelReply, err error) {

	reply = &pb_v1.GateVisitorDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	err = checkGateVisitorDelParam(req)
	if err != nil {
		return nil, err
	}

	g := dbmodel.TGateVisitor{}
	where := [][2]interface{}{}
	where = dbmodel.WhereAdd(where, "id", req.Id)
	if req.Uid > 0 {
		where = dbmodel.WhereAdd(where, "uid", req.Uid)
	}
	err = g.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if g.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if g.OpenTime > 0 {
		return nil, status.Error(10003, "已同行数据不能删除")
	}
	err = g.Delete(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil

}
