package gate_utils

import (
	"fmt"
	"github.com/tidwall/gjson"
	"time"
)

const (
	PersonRegersterTaskName    = "person/create"
	PersonUpdateTaskName       = "person/update"
	PersonDelTaskName          = "person/delete"
	PhotoRegisterByUrlTaskName = "face/createByUrl"
	TakePhotoRegisterTaskName  = "face/takeImg"
	PhotoDelTaskName           = "face/delete"
	DeviceRebootTaskName       = "restartDevice"
	DeviceOpenTaskName         = "device/openDoorControl"
	PhotoUpdateByUrlTaskName   = "face"
)

type PersonRegesterTask struct {
	TaskNo        string             `json:"taskNo"`
	InterfaceName string             `json:"interfaceName"`
	Result        bool               `json:"result"`
	Person        PersonRegesterData `json:"person"`
}

type PersonRegesterData struct {
	Id                    string `json:"id"`
	Name                  string `json:"name"`
	IdCardNum             string `json:"idcardNum"`
	IdNumber              string `json:"iDNumber"`
	FacePermission        int    `json:"facePermission"`
	IdCardPermission      int    `json:"idCardPermission"`
	FaceAndCardPermission int    `json:"faceAndCardPermission"`
	IDPermission          int    `json:"iDPermission"`
	Tag                   string `json:"tag"`
	Phone                 string `json:"phone"`
}

type PersonUpdateTask struct {
	TaskNo        string           `json:"taskNo"`
	InterfaceName string           `json:"interfaceName"`
	Result        bool             `json:"result"`
	Person        PersonUpdateData `json:"person"`
}

type PersonUpdateData struct {
	Id                    string `json:"id"`
	Name                  string `json:"name"`
	IdCardNum             string `json:"idcardNum"`
	IdNumber              string `json:"iDNumber"`
	FacePermission        string `json:"facePermission"`
	IdCardPermission      string `json:"idCardPermission"`
	FaceAndCardPermission string `json:"faceAndCardPermission"`
	IDPermission          string `json:"iDPermission"`
	Tag                   string `json:"tag"`
	Phone                 string `json:"phone"`
}

type PersonDelTask struct {
	TaskNo        string `json:"taskNo"`
	InterfaceName string `json:"interfaceName"`
	Result        bool   `json:"result"`
	Id            string `json:"id"`
}

type PhoneRegisterTask struct {
	TaskNo        string `json:"taskNo"`
	InterfaceName string `json:"interfaceName"`
	Result        bool   `json:"result"`
	PhotoRegisterData
}

type PhotoRegisterData struct {
	PersonId string `json:"personId"`
	FaceId   string `json:"faceId"`
	ImgUrl   string `json:"imgUrl"`
}

type PhotoUpdateTask struct {
	TaskNo        string `json:"taskNo"`
	InterfaceName string `json:"interfaceName"`
	Result        bool   `json:"result"`
	PhotoUpdateData
}

type PhotoUpdateData struct {
	PersonId string `json:"personId"`
	FaceId   string `json:"faceId"`
	Url      string `json:"url"`
}

type PhoneDelTask struct {
	TaskNo        string `json:"taskNo"`
	InterfaceName string `json:"interfaceName"`
	Result        bool   `json:"result"`
	FaceId        string `json:"faceId"`
}

type TaskPhotoRegister struct {
	TaskNo        string `json:"taskNo"`
	InterfaceName string `json:"interfaceName"`
	Result        bool   `json:"result"`
	PersonId      string `json:"personId"`
	ImgUrl        string `json:"imgUrl"`
	FaceId        string `json:"faceId"`
}

type DeviceRebootTask struct {
	TaskNo        string `json:"taskNo"`
	InterfaceName string `json:"interfaceName"`
	Result        bool   `json:"result"`
}

type DeviceOpenTask struct {
	TaskNo        string `json:"taskNo"`
	InterfaceName string `json:"interfaceName"`
	Result        bool   `json:"result"`
	// 1：开门 2：串口 3：韦根 4：表示自定义文字弹
	OType int32 `json:"type"`
}

func CheckYufanIp(ip string, port int64, pass string) bool {
	if port == 0 || ip == "" || pass == "" {
		return false
	}
	h := HttpRequestWithHead{
		Url:     fmt.Sprintf("http://%s:%d/device/config", ip, port),
		Method:  "GET",
		TimeOut: 5 * time.Second,
		Query:   map[string]string{"pass": pass},
	}
	bytes, err := h.Request()
	if err != nil {
		return false
	}
	if gjson.GetBytes(bytes, "code").String() != "" {
		return true
	}
	return false
}
